; manage IRQ/time routines


;==================================
; play music, set IRQ1, set graphics
; for scroll
;==================================
irqR0:
      lda  xScroll
      and  #$07
      ora  #$10
      sta  $D016                 ; set scroll for x as needed    

      ldx  #1
      stx  $d019                 ; acknowledge the raster interrupt
      lda  #<irqR1
      sta  $0314
      lda  #>irqR1
      sta  $0315
      lda  #RASTER1
      sta  $D012                 ; Reading/Writing IRQ balance value

      lda  jumpDelay
      beq  pMusic

      dec  jumpDelay             ; decrease the delay for next jump
pMusic:
      jsr  playMusic    
      
      jsr  diamondAnim           ; animation for diamond
      jsr  forkAnim              ; animation for fork

                                 ; increment time for bridge
      inc  timeBr
      lda  timeBr
      cmp  #T_BRIDGE
      bne  skipTB

      lda  #0
      sta  timeBr
skipTB:

      jmp  $ea81

;=================================
; set IRQ0, set graphics for title
; screen, let performs scrolling
;=================================
irqR1:
      ldx  #1
      stx  $d019                  ; acknowledge the raster interrupt     

      lda  #<irqR0
      sta  $0314

      lda  #$00
      sta  $D016                  ; set not scroll for x (here it did not get flickers)

      lda  #>irqR0
      sta  $0315
      lda  #RASTER0
      sta  $D012                  ; Reading/Writing IRQ balance value

      lda  #1
      sta  GO

      jmp  $ea81 
      
;==================================
; play music, set IRQ3, set graphics
; modo for logo
; a Bar color by iAN CooG/HokutoForce
; routine is used
;==================================
irqR2:      
      ldx #$00                   ; 2
      lda #25                    ; 2
start0
      cmp $d012                  ; 4
      bne start0                 ; 3
                                 ; [from 4 to 9]
      ldx  #1                    ; 2 [4-5]
      stx  $d019                 ; 4 [6-9] acknowledge the raster interrupt
      lda  #<irqR3               ; 2 [10-11]
      sta  $0314                 ; 4 [12-15]     
      lda  #>irqR3               ; 2 [16-17]    
      sta  $0315                 ; 4 [18-21] 
      lda  #RASTER3              ; 2 [22-23]         
      sta  $D012                 ; 4 [24-27] Reading/Writing IRQ balance value
      
      inc  dummy                 ; 6 [28-33]
      dec  dummy                 ; 6 [34-39]
      inc  dummy                 ; 6 [40-45]
      dec  dummy                 ; 6 [46-51]
      inc  dummy                 ; 6 [52-57]                                   
                                       
loop0
      ldy timer0,x
      lda color0,x
      sta $d020
      sta $d021


delay0
        dey
        bne delay0
        inx
        cpx #16;#(endcolor-color)
        bne loop0

        lda  #0
        sta  53280
        sta  53281 
        
        jsr  enableGraphics
        
;rotating raster : down-up
        lda color0
        sta $02
        ldx #00
rotr0   lda color0+1,x
        sta color0  ,x
        inx
        cpx #(endcolor0-color0-1)
        bne rotr0
        lda $02
        sta endcolor0-2                              
      
      jsr  playMusic   
      jsr  colorEffect         
      jmp  $ea81

;=================================
; set IRQ2, set text mode 
; a Bar color by iAN CooG/HokutoForce
; routine is used
;=================================
irqR3: 
      jsr  disableGraphics
      ldx #$00                   
      lda #$83+8*3               
start:
      cmp $d012                  ; 4 [54-57] [59-62]
      bne start                  ; 3 [58-60] [63-2]
                                 ; -- bad line         
      nop                        ; 2 [3-4]   [9-10]  wait some cycles
      nop                        ; 2 [5-6]   [55-56] to make rasters more stable
      nop                        ; 2 [7-8]   [57-58]
      nop                        ; 2 [9-10]  [59-60]
      nop                        ; 2 [11-55] [61-62] 
      nop                        ; 2 [56-57] [63-1]
loop
      ldy timer,x                ; 4 [58-61] [2-5]
      lda color,x                ; 4 [62-3]  [6-9]
      sta $d020                  ; 4 [4-7]   [10-13]
      sta $d021                  ; 4 [8-11]  [14-17]

delay
      dey                        ; 2 [12-13] [18-19] 
      bne delay                  ; 2 [14-15] [20-21]
                                 ;   ..                       
      inx                        
      cpx #16                    
      bne loop                   

      lda  #0
      sta  53280
      sta  53281 
      
      ;jsr  disableGraphics        
        
;rotating raster : up-down
      lda endcolor-2
      sta $fe
      ldx #(endcolor-color-2)
rotr  lda color-1,x
      sta color  ,x
      dex
      bne rotr
      lda $fe
      sta color

      ldx  #1
      stx  $d019                  ; acknowledge the raster interrupt
          
      
      lda  #<irqR2
      sta  $0314            
      lda  #>irqR2
      sta  $0315
      lda  #RASTER2
      sta  $D012                  ; Reading/Writing IRQ balance value

      jmp  $ea81 
      
      
;=================================
; Dec time of 1 sec. after 50
; frames
;=================================
decTime:
      lda  time
      beq  noDecTime
      inc  timeDl
      lda  timeDl
      cmp  #50                    ; 50 frames (pal)=1 second
      bcc  noDecTime
      lda  #$00
      sta  timeDl                 ; reset time frame (delay) conter
      sed
      lda  time
      sec
      sbc  #$01                   ; sub 1 second (in decimal mode)
      sta  time
      cld
      bne  noDecTime

      ; time out
      lda  #1                     ; we must died as out of time
      sta  status
noDecTime:
      rts


;==================================
; Wait for IRQ before going on
;==================================
waitIRQ:
      lda  GO
      beq  waitIRQ
      dec  GO
      rts
      
      
;================================
; waste time (X=amount)
;================================      
wasteTime      
      jsr  waitIRQ
      dex
      bne  wasteTime      
      rts
      
;==================================      
; install interrupt routine
;==================================
installIRQ:
      sei
      lda  #0
      sta  GO
      
      lda  #<irqR0
      sta  $0314
      lda  #>irqR0
      sta  $0315
                                 ; disable cia interrupt
      ldx  #$00
      stx  $DC0E                 ; Control register A of CIA #1
      inx
      stx  $D01A                 ; IRQ mask register
      cli
                                 ; use VIC interrupt
      stx  $D019                 ; Interrupt indicator register
      lda  #RASTER0
      sta  $D012                  ; Reading/Writing IRQ balance value
      lda  #$1B
      sta  $D011                  ; VIC control register      
      rts      
      
;==================================      
; install interrupt routine for logo
;==================================
installLogoIRQ:
      sei
      lda  #<irqR2
      sta  $0314
      lda  #>irqR2
      sta  $0315
                                 ; disable cia interrupt
      ldx  #$00
      stx  $DC0E                 ; Control register A of CIA #1
      inx
      stx  $D01A                 ; IRQ mask register
      cli
                                 ; use VIC interrupt
      stx  $D019                 ; Interrupt indicator register
      lda  #RASTER2
      sta  $D012                 ; Reading/Writing IRQ balance value
      lda  #$1B
      sta  $D011                 ; VIC control register      
      rts          

      
;timer = $0700      
color0  ; italian style r00lz;)
       .byte $02,$02,$02,$0a,$00,$02,$0a,$0f,$00
       .byte $02,$0a,$0f,$01,$00,$02,$0a,$0f,$01
       .byte $01,$01,$0f,$0d,$05,$00,$01,$0f,$0d
       .byte $05,$00,$0f,$0d,$05,$00,$0d,$05,$05,$05,$0
endcolor0

timer0:
       .byte $08, $08, $08, $08, $08, $08, $08, $09
       .byte $08, $08, $08, $08, $08, $08, $08, $0a

color  ; italian style r00lz;)
       .byte $02,$02,$02,$0a,$00,$02,$0a,$0f,$00
       .byte $02,$0a,$0f,$01,$00,$02,$0a,$0f,$01
       .byte $01,$01,$0f,$0d,$05,$00,$01,$0f,$0d
       .byte $05,$00,$0f,$0d,$05,$00,$0d,$05,$05,$05,$0
endcolor    
      
timer: .byte $08, $08, $08, $08, $08, $08, $08, $01
       .byte $08, $08, $08, $08, $08, $08, $08, $01
       
dummy: .byte $00       

  